/********************************************************************************
** Form generated from reading UI file 'configgame.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGGAME_H
#define UI_CONFIGGAME_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_ConfigGame
{
public:
    QVBoxLayout *verticalLayout;
    QCheckBox *kcfg_PrintMulti;
    QSpacerItem *verticalSpacer;
    QCheckBox *kcfg_ShowHighlights;
    QCheckBox *kcfg_HighlightSameValues;
    QCheckBox *kcfg_ShowErrors;
    QCheckBox *kcfg_ShowHighlights3D;
    QSpacerItem *spacer_3;
    QHBoxLayout *horizontalLayout_5;
    QSpinBox *kcfg_MathdokuSize;
    QLabel *label_4;
    QSpacerItem *verticalSpacer_2;
    QLabel *label_2;
    QHBoxLayout *horizontalLayout;
    QSlider *kcfg_OverallSize3D;
    QLabel *label_6;
    QHBoxLayout *horizontalLayout_1;
    QSlider *kcfg_SelectionSize3D;
    QLabel *label;
    QHBoxLayout *horizontalLayout_2;
    QSlider *kcfg_HighlightedSize3D;
    QLabel *label_3;
    QHBoxLayout *horizontalLayout_4;
    QSlider *kcfg_OuterCellSize3D;
    QLabel *label_5;
    QHBoxLayout *horizontalLayout_3;
    QCheckBox *kcfg_DarkenOuterCells3D;
    QSpacerItem *horizontalSpacer;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *ConfigGame)
    {
        if (ConfigGame->objectName().isEmpty())
            ConfigGame->setObjectName("ConfigGame");
        ConfigGame->resize(462, 521);
        verticalLayout = new QVBoxLayout(ConfigGame);
        verticalLayout->setObjectName("verticalLayout");
        kcfg_PrintMulti = new QCheckBox(ConfigGame);
        kcfg_PrintMulti->setObjectName("kcfg_PrintMulti");

        verticalLayout->addWidget(kcfg_PrintMulti);

        verticalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout->addItem(verticalSpacer);

        kcfg_ShowHighlights = new QCheckBox(ConfigGame);
        kcfg_ShowHighlights->setObjectName("kcfg_ShowHighlights");

        verticalLayout->addWidget(kcfg_ShowHighlights);

        kcfg_HighlightSameValues = new QCheckBox(ConfigGame);
        kcfg_HighlightSameValues->setObjectName("kcfg_HighlightSameValues");

        verticalLayout->addWidget(kcfg_HighlightSameValues);

        kcfg_ShowErrors = new QCheckBox(ConfigGame);
        kcfg_ShowErrors->setObjectName("kcfg_ShowErrors");

        verticalLayout->addWidget(kcfg_ShowErrors);

        kcfg_ShowHighlights3D = new QCheckBox(ConfigGame);
        kcfg_ShowHighlights3D->setObjectName("kcfg_ShowHighlights3D");

        verticalLayout->addWidget(kcfg_ShowHighlights3D);

        spacer_3 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout->addItem(spacer_3);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        horizontalLayout_5->setContentsMargins(-1, 0, -1, 0);
        kcfg_MathdokuSize = new QSpinBox(ConfigGame);
        kcfg_MathdokuSize->setObjectName("kcfg_MathdokuSize");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(200);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(kcfg_MathdokuSize->sizePolicy().hasHeightForWidth());
        kcfg_MathdokuSize->setSizePolicy(sizePolicy);
        kcfg_MathdokuSize->setMinimumSize(QSize(84, 0));
        kcfg_MathdokuSize->setMinimum(3);
        kcfg_MathdokuSize->setMaximum(9);
        kcfg_MathdokuSize->setValue(6);

        horizontalLayout_5->addWidget(kcfg_MathdokuSize);

        label_4 = new QLabel(ConfigGame);
        label_4->setObjectName("label_4");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy1);

        horizontalLayout_5->addWidget(label_4);


        verticalLayout->addLayout(horizontalLayout_5);

        verticalSpacer_2 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout->addItem(verticalSpacer_2);

        label_2 = new QLabel(ConfigGame);
        label_2->setObjectName("label_2");
        QFont font;
        font.setPointSize(13);
        label_2->setFont(font);

        verticalLayout->addWidget(label_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        kcfg_OverallSize3D = new QSlider(ConfigGame);
        kcfg_OverallSize3D->setObjectName("kcfg_OverallSize3D");
        sizePolicy.setHeightForWidth(kcfg_OverallSize3D->sizePolicy().hasHeightForWidth());
        kcfg_OverallSize3D->setSizePolicy(sizePolicy);
        kcfg_OverallSize3D->setMinimum(7);
        kcfg_OverallSize3D->setMaximum(13);
        kcfg_OverallSize3D->setSingleStep(1);
        kcfg_OverallSize3D->setPageStep(1);
        kcfg_OverallSize3D->setValue(10);
        kcfg_OverallSize3D->setSliderPosition(10);
        kcfg_OverallSize3D->setOrientation(Qt::Horizontal);
        kcfg_OverallSize3D->setInvertedAppearance(false);
        kcfg_OverallSize3D->setTickPosition(QSlider::TicksBelow);
        kcfg_OverallSize3D->setTickInterval(1);

        horizontalLayout->addWidget(kcfg_OverallSize3D);

        label_6 = new QLabel(ConfigGame);
        label_6->setObjectName("label_6");
        sizePolicy1.setHeightForWidth(label_6->sizePolicy().hasHeightForWidth());
        label_6->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(label_6);


        verticalLayout->addLayout(horizontalLayout);

        horizontalLayout_1 = new QHBoxLayout();
        horizontalLayout_1->setObjectName("horizontalLayout_1");
        kcfg_SelectionSize3D = new QSlider(ConfigGame);
        kcfg_SelectionSize3D->setObjectName("kcfg_SelectionSize3D");
        sizePolicy.setHeightForWidth(kcfg_SelectionSize3D->sizePolicy().hasHeightForWidth());
        kcfg_SelectionSize3D->setSizePolicy(sizePolicy);
        kcfg_SelectionSize3D->setMinimum(7);
        kcfg_SelectionSize3D->setMaximum(13);
        kcfg_SelectionSize3D->setSingleStep(1);
        kcfg_SelectionSize3D->setPageStep(1);
        kcfg_SelectionSize3D->setValue(7);
        kcfg_SelectionSize3D->setSliderPosition(7);
        kcfg_SelectionSize3D->setOrientation(Qt::Horizontal);
        kcfg_SelectionSize3D->setInvertedAppearance(false);
        kcfg_SelectionSize3D->setTickPosition(QSlider::TicksBelow);
        kcfg_SelectionSize3D->setTickInterval(1);

        horizontalLayout_1->addWidget(kcfg_SelectionSize3D);

        label = new QLabel(ConfigGame);
        label->setObjectName("label");
        sizePolicy1.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy1);

        horizontalLayout_1->addWidget(label);


        verticalLayout->addLayout(horizontalLayout_1);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        kcfg_HighlightedSize3D = new QSlider(ConfigGame);
        kcfg_HighlightedSize3D->setObjectName("kcfg_HighlightedSize3D");
        sizePolicy.setHeightForWidth(kcfg_HighlightedSize3D->sizePolicy().hasHeightForWidth());
        kcfg_HighlightedSize3D->setSizePolicy(sizePolicy);
        kcfg_HighlightedSize3D->setMinimum(7);
        kcfg_HighlightedSize3D->setMaximum(13);
        kcfg_HighlightedSize3D->setPageStep(1);
        kcfg_HighlightedSize3D->setValue(7);
        kcfg_HighlightedSize3D->setOrientation(Qt::Horizontal);
        kcfg_HighlightedSize3D->setTickPosition(QSlider::TicksBelow);
        kcfg_HighlightedSize3D->setTickInterval(1);

        horizontalLayout_2->addWidget(kcfg_HighlightedSize3D);

        label_3 = new QLabel(ConfigGame);
        label_3->setObjectName("label_3");
        sizePolicy1.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy1);

        horizontalLayout_2->addWidget(label_3);


        verticalLayout->addLayout(horizontalLayout_2);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        kcfg_OuterCellSize3D = new QSlider(ConfigGame);
        kcfg_OuterCellSize3D->setObjectName("kcfg_OuterCellSize3D");
        sizePolicy.setHeightForWidth(kcfg_OuterCellSize3D->sizePolicy().hasHeightForWidth());
        kcfg_OuterCellSize3D->setSizePolicy(sizePolicy);
        kcfg_OuterCellSize3D->setMinimum(4);
        kcfg_OuterCellSize3D->setMaximum(10);
        kcfg_OuterCellSize3D->setPageStep(1);
        kcfg_OuterCellSize3D->setValue(4);
        kcfg_OuterCellSize3D->setOrientation(Qt::Horizontal);
        kcfg_OuterCellSize3D->setTickPosition(QSlider::TicksBelow);
        kcfg_OuterCellSize3D->setTickInterval(1);

        horizontalLayout_4->addWidget(kcfg_OuterCellSize3D);

        label_5 = new QLabel(ConfigGame);
        label_5->setObjectName("label_5");
        sizePolicy1.setHeightForWidth(label_5->sizePolicy().hasHeightForWidth());
        label_5->setSizePolicy(sizePolicy1);

        horizontalLayout_4->addWidget(label_5);


        verticalLayout->addLayout(horizontalLayout_4);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        kcfg_DarkenOuterCells3D = new QCheckBox(ConfigGame);
        kcfg_DarkenOuterCells3D->setObjectName("kcfg_DarkenOuterCells3D");

        horizontalLayout_3->addWidget(kcfg_DarkenOuterCells3D);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout_3);

        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(spacerItem);

#if QT_CONFIG(shortcut)
        label_6->setBuddy(kcfg_OverallSize3D);
        label->setBuddy(kcfg_SelectionSize3D);
        label_3->setBuddy(kcfg_HighlightedSize3D);
        label_5->setBuddy(kcfg_OuterCellSize3D);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(kcfg_ShowHighlights, kcfg_ShowErrors);
        QWidget::setTabOrder(kcfg_ShowErrors, kcfg_ShowHighlights3D);
        QWidget::setTabOrder(kcfg_ShowHighlights3D, kcfg_SelectionSize3D);
        QWidget::setTabOrder(kcfg_SelectionSize3D, kcfg_HighlightedSize3D);
        QWidget::setTabOrder(kcfg_HighlightedSize3D, kcfg_OuterCellSize3D);
        QWidget::setTabOrder(kcfg_OuterCellSize3D, kcfg_DarkenOuterCells3D);

        retranslateUi(ConfigGame);

        QMetaObject::connectSlotsByName(ConfigGame);
    } // setupUi

    void retranslateUi(QWidget *ConfigGame)
    {
#if QT_CONFIG(tooltip)
        kcfg_PrintMulti->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        kcfg_PrintMulti->setText(tr2i18n("When printing, try to fit more than one puzzle on a page", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_ShowHighlights->setToolTip(tr2i18n("Highlight the row, column and block that contain the selected cell", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        kcfg_ShowHighlights->setWhatsThis(tr2i18n("Highlight the row, column and block that contain the selected cell. Blocks may be square or irregular (as in jigsaw-type or XSudoku puzzles).", nullptr));
#endif // QT_CONFIG(whatsthis)
        kcfg_ShowHighlights->setText(tr2i18n("Show highlights in 2-D puzzles", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_HighlightSameValues->setToolTip(tr2i18n("Highlight cells containing the currently selected value", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        kcfg_HighlightSameValues->setWhatsThis(tr2i18n("When you select a value, all other cells containing the same value will be highlighted. This can help you quickly identify patterns and potential conflicts.", nullptr));
#endif // QT_CONFIG(whatsthis)
        kcfg_HighlightSameValues->setText(tr2i18n("Highlight cells containing the currently selected value", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_ShowErrors->setToolTip(tr2i18n("Highlight in red any cells whose values break the Sudoku rules.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        kcfg_ShowErrors->setWhatsThis(tr2i18n("Highlight in red any cells whose values break the Sudoku rules.", nullptr));
#endif // QT_CONFIG(whatsthis)
        kcfg_ShowErrors->setText(tr2i18n("Show errors", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_ShowHighlights3D->setToolTip(tr2i18n("Highlight the planes (blocks) that contain the selected cell.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        kcfg_ShowHighlights3D->setWhatsThis(tr2i18n("Highlight the planes (blocks) that contain the selected cell.", nullptr));
#endif // QT_CONFIG(whatsthis)
        kcfg_ShowHighlights3D->setText(tr2i18n("Show highlights in 3-D puzzles", nullptr));
        label_4->setText(tr2i18n("Mathdoku puzzle size (3-9)", nullptr));
        label_2->setText(tr2i18n("Settings for 3-D Puzzles Only", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_OverallSize3D->setToolTip(tr2i18n("Set the relative size of 3-D cells and the spaces between them.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        kcfg_OverallSize3D->setWhatsThis(tr2i18n("Set the relative size of 3-D cells and the spaces between them.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        label_6->setToolTip(tr2i18n("Set the relative size of 3-D cells and the spaces between them.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        label_6->setWhatsThis(tr2i18n("Set the relative size of 3-D cells and the spaces between them.", nullptr));
#endif // QT_CONFIG(whatsthis)
        label_6->setText(tr2i18n("Overall size of 3-D cells", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_SelectionSize3D->setToolTip(tr2i18n("Shrink or expand a cell by up to 30% when it is selected.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        kcfg_SelectionSize3D->setWhatsThis(tr2i18n("Shrink or expand a cell by up to 30% when it is selected.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        label->setToolTip(tr2i18n("Shrink or expand a cell by up to 30% when it is selected.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        label->setWhatsThis(tr2i18n("Shrink or expand a cell by up to 30% when it is selected.", nullptr));
#endif // QT_CONFIG(whatsthis)
        label->setText(tr2i18n("Size of selected 3-D cell", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_HighlightedSize3D->setToolTip(tr2i18n("Shrink or expand cells by up to 30% when they are highlighted.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        kcfg_HighlightedSize3D->setWhatsThis(tr2i18n("Shrink or expand cells by up to 30% when they are highlighted.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        label_3->setToolTip(tr2i18n("Shrink or expand cells by up to 30% when they are highlighted.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        label_3->setWhatsThis(tr2i18n("Shrink or expand cells by up to 30% when they are highlighted.", nullptr));
#endif // QT_CONFIG(whatsthis)
        label_3->setText(tr2i18n("Size of highlighted 3-D cells", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_OuterCellSize3D->setToolTip(tr2i18n("Shrink cells by up to 60% if they are not in an NxNxN cube that contains the selected cell.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        kcfg_OuterCellSize3D->setWhatsThis(tr2i18n("Shrink cells by up to 60% if they are not in an NxNxN cube that contains the selected cell. This is useful if a 3-D puzzle has more than one cube.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        label_5->setToolTip(tr2i18n("Shrink cells by up to 60% if they are not in an NxNxN cube that contains the selected cell.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        label_5->setWhatsThis(tr2i18n("Shrink cells by up to 60% if they are not in an NxNxN cube that contains the selected cell. This is useful if a 3-D puzzle has more than one cube.", nullptr));
#endif // QT_CONFIG(whatsthis)
        label_5->setText(tr2i18n("Size of 3-D cells outside the selected volume", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_DarkenOuterCells3D->setToolTip(tr2i18n("Darken cells if they are not in an NxNxN cube that contains the selected cell.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        kcfg_DarkenOuterCells3D->setWhatsThis(tr2i18n("Shrink cells by up to 60% if they are not in an NxNxN cube that contains the selected cell. This is useful if a 3-D puzzle has more than one cube.", nullptr));
#endif // QT_CONFIG(whatsthis)
        kcfg_DarkenOuterCells3D->setText(tr2i18n("Darken 3-D cells outside the selected volume", nullptr));
        (void)ConfigGame;
    } // retranslateUi

};

namespace Ui {
    class ConfigGame: public Ui_ConfigGame {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CONFIGGAME_H

