/********************************************************************************
** Form generated from reading UI file 'welcomescreen.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_WELCOMESCREEN_H
#define UI_WELCOMESCREEN_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QListView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_WelcomeScreen
{
public:
    QHBoxLayout *hboxLayout;
    QGroupBox *groupBox1;
    QVBoxLayout *vboxLayout;
    QListView *gameListWidget;
    QHBoxLayout *hboxLayout1;
    QPushButton *getNewGameButton;
    QSpacerItem *spacerItem;
    QPushButton *startEmptyButton;
    QPushButton *puzzleGeneratorButton;

    void setupUi(QWidget *WelcomeScreen)
    {
        if (WelcomeScreen->objectName().isEmpty())
            WelcomeScreen->setObjectName("WelcomeScreen");
        WelcomeScreen->resize(512, 512);
        hboxLayout = new QHBoxLayout(WelcomeScreen);
        hboxLayout->setObjectName("hboxLayout");
        groupBox1 = new QGroupBox(WelcomeScreen);
        groupBox1->setObjectName("groupBox1");
        vboxLayout = new QVBoxLayout(groupBox1);
        vboxLayout->setObjectName("vboxLayout");
        gameListWidget = new QListView(groupBox1);
        gameListWidget->setObjectName("gameListWidget");

        vboxLayout->addWidget(gameListWidget);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName("hboxLayout1");
        getNewGameButton = new QPushButton(groupBox1);
        getNewGameButton->setObjectName("getNewGameButton");

        hboxLayout1->addWidget(getNewGameButton);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacerItem);

        startEmptyButton = new QPushButton(groupBox1);
        startEmptyButton->setObjectName("startEmptyButton");

        hboxLayout1->addWidget(startEmptyButton);

        puzzleGeneratorButton = new QPushButton(groupBox1);
        puzzleGeneratorButton->setObjectName("puzzleGeneratorButton");

        hboxLayout1->addWidget(puzzleGeneratorButton);


        vboxLayout->addLayout(hboxLayout1);


        hboxLayout->addWidget(groupBox1);


        retranslateUi(WelcomeScreen);

        puzzleGeneratorButton->setDefault(true);


        QMetaObject::connectSlotsByName(WelcomeScreen);
    } // setupUi

    void retranslateUi(QWidget *WelcomeScreen)
    {
        groupBox1->setTitle(tr2i18n("Choose your Game", nullptr));
        getNewGameButton->setText(tr2i18n("Get New", nullptr));
        startEmptyButton->setText(tr2i18n("Enter In A Puzzle", nullptr));
        puzzleGeneratorButton->setText(tr2i18n("Generate A Puzzle", nullptr));
        (void)WelcomeScreen;
    } // retranslateUi

};

namespace Ui {
    class WelcomeScreen: public Ui_WelcomeScreen {};
} // namespace Ui

QT_END_NAMESPACE

#endif // WELCOMESCREEN_H

