/****************************************************************************
** Resource object code
**
** Created by: The Resource Compiler for Qt version 6.12.0
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#ifdef _MSC_VER
// disable informational message "function ... selected for automatic inline expansion"
#pragma warning (disable: 4711)
#endif

static const unsigned char qt_resource_data[] = {
  // ksudokuui.rc
  0x0,0x0,0x2,0x4c,
  0x28,
  0xb5,0x2f,0xfd,0x60,0x65,0x7,0x15,0x12,0x0,0xb6,0x97,0x48,0x1f,0x30,0x73,0x1e,
  0xcc,0x2a,0x37,0x32,0x49,0x6a,0x48,0x52,0x69,0x9b,0x56,0x1c,0xdf,0x62,0xd7,0xbf,
  0x6d,0x32,0xae,0x75,0x44,0xe9,0xa,0x5,0x15,0x79,0x41,0x5,0x42,0x0,0x42,0x0,
  0x3d,0x0,0x1b,0xf7,0xb0,0x32,0x9c,0x6c,0xdf,0xc5,0xea,0x83,0x86,0x9b,0x41,0xa7,
  0xd3,0x71,0x4e,0x41,0xad,0xd5,0x34,0xc6,0x4f,0x4,0x83,0xda,0xdd,0x38,0xbc,0xbf,
  0x69,0x8c,0xb6,0x78,0x32,0x79,0xe3,0x83,0xa2,0x46,0x4b,0x8,0x22,0x96,0x4b,0x63,
  0xc4,0x29,0xa1,0x53,0xb3,0x59,0x23,0xc,0xa,0x8d,0x31,0x5d,0x7e,0xed,0x8b,0x55,
  0x96,0xa7,0xf1,0x2,0x91,0x51,0x8e,0x7,0x70,0x4f,0xb2,0xa0,0x16,0x7,0x22,0xeb,
  0x69,0xfb,0x22,0xd6,0x62,0xeb,0x58,0x5b,0xf2,0xc9,0x0,0x50,0x54,0xf9,0x71,0x67,
  0x4b,0xf,0x82,0xa2,0x6,0xdf,0x33,0x60,0xd0,0x74,0x69,0x99,0x72,0xca,0x2b,0xe4,
  0x37,0x35,0xa5,0x80,0xad,0xd6,0x50,0x15,0x8a,0x84,0x71,0x5,0x2,0xc1,0x70,0xe8,
  0xc8,0xf7,0x28,0x9c,0xbc,0x31,0xdf,0xd5,0xe7,0x73,0xe9,0x94,0xb9,0xf0,0xaa,0xfc,
  0xed,0x94,0x9a,0x9,0x42,0xee,0xda,0x1c,0xce,0x3b,0x5d,0x2e,0xb5,0xbe,0xdc,0x82,
  0x22,0xf7,0xac,0xbc,0x1c,0x3b,0x65,0xa5,0xcf,0x17,0x7f,0x75,0xcb,0x95,0x69,0x57,
  0x20,0x7b,0xd5,0xfa,0x2f,0xd3,0x66,0x6b,0x57,0xc7,0x58,0xeb,0xf4,0x9c,0xac,0x89,
  0x5e,0xaf,0x1,0x50,0x24,0x6f,0x74,0xee,0xbd,0x84,0x2f,0x4e,0x99,0x69,0xcb,0xb6,
  0xf2,0xd4,0xad,0x7c,0xa7,0xac,0x3d,0xbb,0xe7,0x2a,0x67,0x98,0x34,0x25,0x57,0xee,
  0x96,0x7,0xcd,0xb6,0x2c,0x87,0xc0,0xd9,0xb6,0x3a,0xa5,0xf7,0x7d,0x32,0xa1,0x55,
  0x6b,0xda,0x77,0xe4,0xff,0xae,0x32,0x3c,0xbb,0x7d,0x9c,0xd2,0xda,0xe,0x7c,0xa0,
  0xc1,0x19,0x44,0x37,0xa4,0x4d,0x19,0xc9,0xc6,0x18,0x30,0x84,0x10,0x4,0xd9,0xcd,
  0x3,0xaf,0x3,0x26,0x5e,0xca,0x5,0xe5,0x3,0x96,0x88,0x92,0x9,0xe8,0x8,0xa4,
  0x34,0xd8,0x12,0x18,0xb,0x53,0x51,0x97,0x5,0x0,0xa1,0xd9,0x46,0x36,0xef,0x1a,
  0x7d,0x3b,0xb3,0xda,0xce,0xa8,0xdb,0xb1,0x91,0xe3,0xc6,0x51,0x36,0xe2,0xa7,0x8c,
  0x51,0x6d,0x50,0x99,0x19,0xd3,0x4e,0xad,0x15,0xda,0x59,0xc0,0x1c,0x49,0xaa,0xa9,
  0xe7,0x6a,0x25,0x75,0x7f,0x26,0x8,0x93,0xa6,0x3e,0x53,0xfd,0x4f,0x66,0x15,0xa,
  0x3a,0x69,0x98,0x11,0xac,0x26,0x77,0xb0,0xc4,0x33,0x7,0xa9,0x49,0x84,0x31,0xb0,
  0x0,0xb1,0x43,0x5f,0xe5,0x5e,0x6b,0xa4,0x26,0x21,0xc0,0x23,0x5,0x62,0xde,0xa3,
  0x44,0xb6,0xcf,0x29,0x40,0x4c,0xf0,0xe,0xdb,0xad,0xd5,0xb5,0x3,0x84,0xf,0xb0,
  0x3,0x21,0x44,0x5e,0xab,0xb3,0x1b,0x7a,0x6d,0xcd,0xb3,0xbe,0x7d,0x65,0x3e,0xe7,
  0xd6,0x2a,0xe8,0xb5,0xcd,0xa,0xe7,0xd0,0x36,0x45,0x82,0x99,0x8,0xb7,0x4d,0x68,
  0x23,0x22,0xc,0xe,0xb3,0x8d,0xe6,0x5c,0x4e,0x56,0xb5,0x4d,0x0,0x5b,0x26,0xae,
  0xe9,0xce,0x5b,0xe0,0x1c,0xc1,0xd4,0x26,0x55,0x0,0xa1,0x5f,0xa9,0xdf,0x75,0x22,
  0xf5,0xf5,0x6e,0x9b,0x16,0x7d,0xd2,0x3f,0xc7,0x15,0x18,0x31,0xc0,0xa6,0x80,0x22,
  0xca,0x3c,0xb0,0x2c,0x31,0x96,0x24,0xa0,0xf,0x99,0x9,0xc4,0xa,0x40,0x86,0x81,
  0x88,0x2c,0xab,0x5e,0x16,0xfa,0xe6,0xf7,0x93,0xb0,0xa8,0xe5,0x9,0x64,0x3,0x1,
  0xea,0x5a,0x7a,0x7f,0xd9,0x78,0x91,0x2b,0x44,0xf7,0x90,0xc7,0x25,0x1e,0xd0,0x85,
  0x6,0x90,0xe,0x86,0xcc,0xfb,0x6f,0x2d,0x3,0xa0,0xa,
  
};

static const unsigned char qt_resource_name[] = {
  // kxmlgui5
  0x0,0x8,
  0xf,0x42,0xe5,0x85,
  0x0,0x6b,
  0x0,0x78,0x0,0x6d,0x0,0x6c,0x0,0x67,0x0,0x75,0x0,0x69,0x0,0x35,
    // ksudoku
  0x0,0x7,
  0x2,0xab,0xb6,0xc5,
  0x0,0x6b,
  0x0,0x73,0x0,0x75,0x0,0x64,0x0,0x6f,0x0,0x6b,0x0,0x75,
    // ksudokuui.rc
  0x0,0xc,
  0xc,0x9e,0xf3,0x43,
  0x0,0x6b,
  0x0,0x73,0x0,0x75,0x0,0x64,0x0,0x6f,0x0,0x6b,0x0,0x75,0x0,0x75,0x0,0x69,0x0,0x2e,0x0,0x72,0x0,0x63,
  
};

static const unsigned char qt_resource_struct[] = {
  // :
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x1,
0x0,0x0,0x1,0x9f,0x70,0xc7,0x57,0x20,
  // :/kxmlgui5
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x2,
0x0,0x0,0x1,0x9f,0x70,0xc7,0x57,0x20,
  // :/kxmlgui5/ksudoku
  0x0,0x0,0x0,0x16,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x3,
0x0,0x0,0x1,0x9f,0x70,0xc7,0x57,0x20,
  // :/kxmlgui5/ksudoku/ksudokuui.rc
  0x0,0x0,0x0,0x2a,0x0,0x4,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x1,0x9f,0x70,0xc7,0x57,0x20,

};

#ifdef QT_NAMESPACE
#  define QT_RCC_PREPEND_NAMESPACE(name) ::QT_NAMESPACE::name
#  define QT_RCC_MANGLE_NAMESPACE0(x) x
#  define QT_RCC_MANGLE_NAMESPACE1(a, b) a##_##b
#  define QT_RCC_MANGLE_NAMESPACE2(a, b) QT_RCC_MANGLE_NAMESPACE1(a,b)
#  define QT_RCC_MANGLE_NAMESPACE(name) QT_RCC_MANGLE_NAMESPACE2( \
        QT_RCC_MANGLE_NAMESPACE0(name), QT_RCC_MANGLE_NAMESPACE0(QT_NAMESPACE))
#else
#   define QT_RCC_PREPEND_NAMESPACE(name) name
#   define QT_RCC_MANGLE_NAMESPACE(name) name
#endif

#if defined(QT_INLINE_NAMESPACE)
inline namespace QT_NAMESPACE {
#elif defined(QT_NAMESPACE)
namespace QT_NAMESPACE {
#endif

bool qRegisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
bool qUnregisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);

#if defined(__ELF__) || defined(__APPLE__)
static inline unsigned char qResourceFeatureZstd()
{
    extern const unsigned char qt_resourceFeatureZstd;
    return qt_resourceFeatureZstd;
}
#else
unsigned char qResourceFeatureZstd();
#endif

#ifdef QT_NAMESPACE
}
#endif

int QT_RCC_MANGLE_NAMESPACE(qInitResources_ksudoku)();
int QT_RCC_MANGLE_NAMESPACE(qInitResources_ksudoku)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qRegisterResourceData)
        (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_ksudoku)();
int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_ksudoku)()
{
    int version = 3;
    version += QT_RCC_PREPEND_NAMESPACE(qResourceFeatureZstd());
    QT_RCC_PREPEND_NAMESPACE(qUnregisterResourceData)
       (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

#ifdef __clang__
#   pragma clang diagnostic push
#   pragma clang diagnostic ignored "-Wexit-time-destructors"
#endif

namespace {
   struct initializer {
       initializer() { QT_RCC_MANGLE_NAMESPACE(qInitResources_ksudoku)(); }
       ~initializer() { QT_RCC_MANGLE_NAMESPACE(qCleanupResources_ksudoku)(); }
   } dummy;
}

#ifdef __clang__
#   pragma clang diagnostic pop
#endif
