/****************************************************************************
** Meta object code from reading C++ file 'ksudoku.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../ksudoku-26.07.80/src/gui/ksudoku.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ksudoku.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7KSudokuE_t {};
} // unnamed namespace

template <> constexpr inline auto KSudoku::qt_create_metaobjectdata<qt_meta_tag_ZN7KSudokuE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KSudoku",
        "settingsChanged",
        "",
        "onCompleted",
        "isCorrect",
        "required",
        "withHelp",
        "showWelcomeScreen",
        "startGame",
        "::ksudoku::Game",
        "game",
        "endCurrentGame",
        "queryClose",
        "handleCurrentDifficultyLevelChanged",
        "const KGameDifficultyLevel*",
        "level",
        "symmetryChanged",
        "symmetry",
        "gameNew",
        "gameRestart",
        "gameOpen",
        "gameSave",
        "gameSaveAs",
        "gamePrint",
        "gameExport",
        "gamePause",
        "undo",
        "redo",
        "push",
        "pop",
        "giveHint",
        "autoSolve",
        "dubPuzzle",
        "genMultiple",
        "optionsPreferences",
        "updateSettings",
        "changeCaption",
        "text",
        "onModified",
        "isModified",
        "enableMessages",
        "changeEvent",
        "QEvent*",
        "event",
        "setTimerDisplay"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'settingsChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onCompleted'
        QtMocHelpers::SlotData<void(bool, QTime, bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 }, { QMetaType::QTime, 5 }, { QMetaType::Bool, 6 },
        }}),
        // Slot 'onCompleted'
        QtMocHelpers::SlotData<void(bool, QTime)>(3, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 4 }, { QMetaType::QTime, 5 },
        }}),
        // Slot 'showWelcomeScreen'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'startGame'
        QtMocHelpers::SlotData<void(const ::ksudoku::Game &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'endCurrentGame'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'queryClose'
        QtMocHelpers::SlotData<bool()>(12, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'handleCurrentDifficultyLevelChanged'
        QtMocHelpers::SlotData<void(const KGameDifficultyLevel *)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'symmetryChanged'
        QtMocHelpers::SlotData<void(int)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 17 },
        }}),
        // Slot 'gameNew'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'gameRestart'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'gameOpen'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'gameSave'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'gameSaveAs'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'gamePrint'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'gameExport'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'gamePause'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'undo'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'redo'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'push'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'pop'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'giveHint'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'autoSolve'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'dubPuzzle'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'genMultiple'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'optionsPreferences'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateSettings'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'changeCaption'
        QtMocHelpers::SlotData<void(const QString &)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 37 },
        }}),
        // Slot 'onModified'
        QtMocHelpers::SlotData<void(bool)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 39 },
        }}),
        // Slot 'enableMessages'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'changeEvent'
        QtMocHelpers::SlotData<void(QEvent *)>(41, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 42, 43 },
        }}),
        // Slot 'setTimerDisplay'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KSudoku, qt_meta_tag_ZN7KSudokuE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KSudoku::staticMetaObject = { {
    QMetaObject::SuperData::link<KXmlGuiWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7KSudokuE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7KSudokuE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7KSudokuE_t>.metaTypes,
    nullptr
} };

void KSudoku::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KSudoku *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->settingsChanged(); break;
        case 1: _t->onCompleted((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QTime>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 2: _t->onCompleted((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QTime>>(_a[2]))); break;
        case 3: _t->showWelcomeScreen(); break;
        case 4: _t->startGame((*reinterpret_cast<std::add_pointer_t<::ksudoku::Game>>(_a[1]))); break;
        case 5: _t->endCurrentGame(); break;
        case 6: { bool _r = _t->queryClose();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->handleCurrentDifficultyLevelChanged((*reinterpret_cast<std::add_pointer_t<const KGameDifficultyLevel*>>(_a[1]))); break;
        case 8: _t->symmetryChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->gameNew(); break;
        case 10: _t->gameRestart(); break;
        case 11: _t->gameOpen(); break;
        case 12: _t->gameSave(); break;
        case 13: _t->gameSaveAs(); break;
        case 14: _t->gamePrint(); break;
        case 15: _t->gameExport(); break;
        case 16: _t->gamePause(); break;
        case 17: _t->undo(); break;
        case 18: _t->redo(); break;
        case 19: _t->push(); break;
        case 20: _t->pop(); break;
        case 21: _t->giveHint(); break;
        case 22: _t->autoSolve(); break;
        case 23: _t->dubPuzzle(); break;
        case 24: _t->genMultiple(); break;
        case 25: _t->optionsPreferences(); break;
        case 26: _t->updateSettings(); break;
        case 27: _t->changeCaption((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 28: _t->onModified((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 29: _t->enableMessages(); break;
        case 30: _t->changeEvent((*reinterpret_cast<std::add_pointer_t<QEvent*>>(_a[1]))); break;
        case 31: _t->setTimerDisplay(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KSudoku::*)()>(_a, &KSudoku::settingsChanged, 0))
            return;
    }
}

const QMetaObject *KSudoku::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KSudoku::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7KSudokuE_t>.strings))
        return static_cast<void*>(this);
    return KXmlGuiWindow::qt_metacast(_clname);
}

int KSudoku::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KXmlGuiWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 32)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 32;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 32)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 32;
    }
    return _id;
}

// SIGNAL 0
void KSudoku::settingsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
