/****************************************************************************
** Meta object code from reading C++ file 'ksudokugame.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../ksudoku-26.07.80/src/gui/ksudokugame.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ksudokugame.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7ksudoku9GameIFaceE_t {};
} // unnamed namespace

template <> constexpr inline auto ksudoku::GameIFace::qt_create_metaobjectdata<qt_meta_tag_ZN7ksudoku9GameIFaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ksudoku::GameIFace",
        "modified",
        "",
        "isModified",
        "completed",
        "isCorrect",
        "required",
        "withHelp",
        "cellChange",
        "index",
        "fullChange",
        "cageChange",
        "cageNum",
        "showLabel",
        "undo",
        "redo",
        "addCheckpoint",
        "undo2Checkpoint"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'modified'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'completed'
        QtMocHelpers::SignalData<void(bool, const QTime &, bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 }, { QMetaType::QTime, 6 }, { QMetaType::Bool, 7 },
        }}),
        // Signal 'cellChange'
        QtMocHelpers::SignalData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Signal 'fullChange'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cageChange'
        QtMocHelpers::SignalData<void(int, bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 }, { QMetaType::Bool, 13 },
        }}),
        // Slot 'undo'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'redo'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'addCheckpoint'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'undo2Checkpoint'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<GameIFace, qt_meta_tag_ZN7ksudoku9GameIFaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ksudoku::GameIFace::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7ksudoku9GameIFaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7ksudoku9GameIFaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7ksudoku9GameIFaceE_t>.metaTypes,
    nullptr
} };

void ksudoku::GameIFace::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GameIFace *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->modified((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->completed((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QTime>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 2: _t->cellChange((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->fullChange(); break;
        case 4: _t->cageChange((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 5: _t->undo(); break;
        case 6: _t->redo(); break;
        case 7: _t->addCheckpoint(); break;
        case 8: _t->undo2Checkpoint(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (GameIFace::*)(bool )>(_a, &GameIFace::modified, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (GameIFace::*)(bool , const QTime & , bool )>(_a, &GameIFace::completed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (GameIFace::*)(int )>(_a, &GameIFace::cellChange, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (GameIFace::*)()>(_a, &GameIFace::fullChange, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (GameIFace::*)(int , bool )>(_a, &GameIFace::cageChange, 4))
            return;
    }
}

const QMetaObject *ksudoku::GameIFace::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ksudoku::GameIFace::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7ksudoku9GameIFaceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ksudoku::GameIFace::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void ksudoku::GameIFace::modified(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void ksudoku::GameIFace::completed(bool _t1, const QTime & _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void ksudoku::GameIFace::cellChange(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void ksudoku::GameIFace::fullChange()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void ksudoku::GameIFace::cageChange(int _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}
QT_WARNING_POP
